# This file contains settings for pytest that are specific to extension-helpers.
# Since we run many of the tests in sub-processes, we need to collect coverage
# data inside each subprocess and then combine it into a single .coverage file.
# To do this we set up a list which run_setup appends coverage objects to.
# This is not intended to be used by packages other than extension-helpers.

import glob
import os

try:
    from coverage import CoverageData
    from coverage import __version__ as coverage_version
except ImportError:
    HAS_COVERAGE = False
    CoverageData = None
else:
    # Set to the major version number
    HAS_COVERAGE = int(coverage_version.split(".")[0])


SUBPROCESS_COVERAGE = []


def pytest_configure(config):
    if HAS_COVERAGE:
        SUBPROCESS_COVERAGE.clear()


def pytest_unconfigure(config):
    if HAS_COVERAGE:
        # Add all files from extension_helpers to make sure we compute the total
        # coverage, not just the coverage of the files that have non-zero
        # coverage.

        lines = {}
        for filename in glob.glob(os.path.join("extension_helpers", "**", "*.py"), recursive=True):
            lines[os.path.abspath(filename)] = []

        for cdata in SUBPROCESS_COVERAGE:
            # For each CoverageData object, we go through all the files and
            # change the filename from one which might be a temporary path
            # to the local filename. We then only keep files that actually
            # exist.
            for filename in cdata.measured_files():
                try:
                    pos = filename.rindex("extension_helpers")
                except ValueError:
                    continue
                short_filename = filename[pos:]
                if os.path.exists(short_filename):
                    lines[os.path.abspath(short_filename)].extend(cdata.lines(filename))

        if HAS_COVERAGE >= 5:
            # Support coverage<5 and >=5; see
            # https://github.com/astropy/extension-helpers/issues/24
            # We create an empty coverage data object
            combined_cdata = CoverageData(suffix="subprocess")
            combined_cdata.add_lines(lines)
            combined_cdata.write()
        else:
            combined_cdata = CoverageData()
            combined_cdata.add_lines(lines)
            combined_cdata.write_file(".coverage.subprocess")



import sys
from configparser import ConfigParser

from ._openmp_helpers import add_openmp_flags_if_available  # noqa: F401
from ._setup_helpers import get_compiler, get_extensions, pkg_config  # noqa: F401
from ._utils import import_file, write_if_different  # noqa: F401
from .version import version as __version__  # noqa: F401


def _finalize_distribution_hook(distribution):
    """
    Entry point for setuptools which allows extension-helpers to be enabled
    from setup.cfg without the need for setup.py.
    """
    import os
    from pathlib import Path

    if sys.version_info >= (3, 11):
        import tomllib
    else:
        import tomli as tomllib

    found_config = False

    config_files = distribution.find_config_files()
    if len(config_files) > 0:
        cfg = ConfigParser()
        cfg.read(config_files[0])
        if cfg.has_option("extension-helpers", "use_extension_helpers"):
            found_config = True
            if cfg.get("extension-helpers", "use_extension_helpers").lower() == "true":
                distribution.ext_modules = get_extensions()

    pyproject = Path(distribution.src_root or os.curdir, "pyproject.toml")
    if pyproject.exists() and not found_config:
        with pyproject.open("rb") as f:
            pyproject_cfg = tomllib.load(f)
            if (
                "tool" in pyproject_cfg
                and "extension-helpers" in pyproject_cfg["tool"]
                and "use_extension_helpers" in pyproject_cfg["tool"]["extension-helpers"]
                and pyproject_cfg["tool"]["extension-helpers"]["use_extension_helpers"]
            ):
                distribution.ext_modules = get_extensions()



import os
import time

import pytest

from .._utils import (
    abi_to_versions,
    get_limited_api_option,
    import_file,
    write_if_different,
)


@pytest.mark.parametrize("path_type", ("str", "path"))
def test_import_file(tmp_path, path_type):
    filepath = tmp_path / "spam.py"
    if path_type == "str":
        filepath = str(filepath)
    with open(filepath, "w") as f:
        f.write("magic = 12345")
    module = import_file(filepath)
    assert module.magic == 12345


@pytest.mark.parametrize("path_type", ("str", "path"))
def test_write_if_different(tmp_path, path_type):
    filepath = tmp_path / "test.txt"
    if path_type == "str":
        filepath = str(filepath)
    write_if_different(filepath, b"abc")
    time1 = os.path.getmtime(filepath)
    time.sleep(0.01)
    write_if_different(filepath, b"abc")
    time2 = os.path.getmtime(filepath)
    assert time2 == time1
    time.sleep(0.01)
    write_if_different(filepath, b"abcd")
    time3 = os.path.getmtime(filepath)
    assert time3 > time1


class TestGetLimitedAPIOption:

    def test_nofiles(self, tmp_path):
        assert get_limited_api_option(tmp_path) is None

    def test_empty_setup_cfg(self, tmp_path):
        (tmp_path / "setup.cfg").write_text("")
        assert get_limited_api_option(tmp_path) is None

    def test_empty_pyproject_toml(self, tmp_path):
        (tmp_path / "pyproject.toml").write_text("")
        assert get_limited_api_option(tmp_path) is None

    def test_setup_cfg(self, tmp_path):

        (tmp_path / "setup.cfg").write_text("[bdist_wheel]\npy_limited_api=cp311")
        assert get_limited_api_option(tmp_path) == "cp311"

        # Make sure things still work even if an empty pyproject.toml file is present

        (tmp_path / "pyproject.toml").write_text("")
        assert get_limited_api_option(tmp_path) == "cp311"

        # And if the pyproject.toml has the right section but not the right option

        (tmp_path / "setup.cfg.toml").write_text("[tool.distutils.bdist_wheel]\nspam=1\n")
        assert get_limited_api_option(tmp_path) == "cp311"

    def test_pyproject(self, tmp_path):

        (tmp_path / "pyproject.toml").write_text(
            '[tool.distutils.bdist_wheel]\npy-limited-api="cp312"\n'
        )
        assert get_limited_api_option(tmp_path) == "cp312"

        # Make sure things still work even if an empty setup.cfg file is present

        (tmp_path / "setup.cfg.toml").write_text("\n")
        assert get_limited_api_option(tmp_path) == "cp312"

        # And if the setup.cfg has the right section but not the right option

        (tmp_path / "setup.cfg.toml").write_text("[bdist_wheel]\nspam=1\n")
        assert get_limited_api_option(tmp_path) == "cp312"


def test_abi_to_versions_invalid():
    assert abi_to_versions("spam") == (None, None)


def test_abi_to_versions_valid():
    assert abi_to_versions("cp39") == ((3, 9), "0x03090000")
    assert abi_to_versions("cp310") == ((3, 10), "0x030A0000")
    assert abi_to_versions("cp311") == ((3, 11), "0x030B0000")
    assert abi_to_versions("cp312") == ((3, 12), "0x030C0000")
    assert abi_to_versions("cp313") == ((3, 13), "0x030D0000")
